'''
    常用的 HTTP 请求——封装了发生错误时重试的功能
'''

import logging
from time import sleep, time

import requests

from .constants import HEADERS, TIMEOUT, STATUS_CODES

# 设置日志
logging.basicConfig(
    level=logging.WARNING,
    format='%(levelname)s %(message)s'
)

# 禁用SLL证书警告
requests.urllib3.disable_warnings()


# 发生错误时再次请求，默认重试 3 次
def retry_get(url, m=3, headers=HEADERS, timeout=TIMEOUT, **kwargs):
    c = 1
    while True:
        try:
            resp = requests.get(url, headers=headers,
                                timeout=timeout, verify=False, **kwargs)
            if resp.status_code in STATUS_CODES:
                return resp
            if c >= m:
                logging.warning(f"[{resp.status_code}]: {url}")
                return False
            c += 1
        except Exception as error:
            if c >= m:
                logging.error(f"{error}: {url}")
                return False
            c += 1
            # 发生延时、最大请求数量等错误时休眠若干秒
            sleep(3)


def retry_post(url, data=None, m=3, headers=HEADERS, timeout=TIMEOUT, **kwargs):
    c = 1
    while True:
        try:
            resp = requests.post(url, data=data, headers=headers,
                                timeout=timeout, verify=False, **kwargs)
            if resp.status_code in STATUS_CODES:
                return resp
            if c >= m:
                logging.warning(f"[{resp.status_code}]: {url}")
                return False
            c += 1
        except Exception as error:
            if c >= m:
                logging.error(f"{error}: {url}")
                return False
            c += 1
            # 发生延时、最大请求数量等错误时休眠若干秒
            sleep(3)


# 下载文件，默认重试 8 次
def download(src, dst, headers=HEADERS, proxies=None, timeout=TIMEOUT):
    resp = retry_get(src, 8, headers, proxies, timeout)
    if resp:
        with open(dst, 'wb') as f:
            f.write(resp.content)
        logging.info(src)


# 将请求数据写入文件，默认将网页标题作为文件名
def save_html(response, fn='default.html'):
    if not fn.endswith('.html'):
        fn = '%s.html' % fn
    with open(fn, 'wb') as fp:
        fp.write(response.content)



'''
    百度图片文字识别接口封装
'''

import base64

from .httplib import HEADERS, retry_get, retry_post


class GeneralOcr(object):
    '''通用文字识别，而非精确识别接口'''

    __client_id = 'yIDSsYZD1uk1OcdOmuaSDYsZ'
    __client_secret = '5rqZhYFgs2VQcLXRNMNeQSWIeqexVajc'

    def __init__(self, client_id=None, client_secret=None, access_token=None):
        self.client_id = client_id or self.__client_id
        self.client_secret = client_id or self.__client_secret
        self.access_token = access_token or self.oauth()

    @classmethod
    def oauth(cls, client_id=None, client_secret=None):
        client_id = client_id or cls.__client_id
        client_secret = client_secret or cls.__client_secret
        oauth_url = f'https://aip.baidubce.com/oauth/2.0/token?grant_type=client_credentials&client_id={client_id}&client_secret={client_secret}'
        oauth_data = retry_get(oauth_url).json()
        return oauth_data['access_token']

    def basic_general(self, image):
        general = f"https://aip.baidubce.com/rest/2.0/ocr/v1/general_basic?access_token={self.access_token}"
        data = {'image': base64.b64encode(image)}
        HEADERS['Content-Type'] = 'application/x-www-form-urlencoded'
        resp = retry_post(general, data=data, headers=HEADERS)
        return resp.json() if resp else ({'words_result': '请求错误'})

    def basic_ocr(self, addr, sep='\n'):
        if addr.startswith('http'):
            if addr.endswith('png') or addr.endswith('jpg'):
                resp = retry_get(addr)
                image = resp.content if resp else b''
            else:
                pass
        else:
            with open(addr, 'rb') as fp:
                image = fp.read()
        _wds = self.basic_general(image).get('words_result', tuple())
        wds = ''
        for _wd in _wds:
            wds += _wd.get('words', '') + sep
        return wds



'''Building a modular crawler template system based on Jinja2'''

import re
from datetime import datetime

from jinja2 import Environment, FileSystemLoader

from .utils import replace_all, replace_ws, retry_get
from .utils.constants import DATETIME_12, WHITESPACE


def render_templatefile(dst, templatefile, **kwargs):
    env = Environment(loader=FileSystemLoader(
        kwargs.pop('templates_folder', './templates')))
    template = env.get_template(templatefile)
    with open(dst, 'w', encoding='utf-8') as fp:
        fp.write(template.render(**kwargs))


# 可根据实际需求修改部分代码
def genspider(home_url, templatefile, dst, spider, **kwargs):
    response = retry_get(home_url)
    if not response:
        raise Exception('Request failed, invalid URL or parameter.')

    try:
        string = response.content.decode('utf8')
    except UnicodeDecodeError:
        string = response.content.decode('gbk')

    source = re.findall(r'(?is)<title>(.*?)</title>', string)[0]

    # 清理标题中的干扰字符
    clear_chars = kwargs.pop('clear_chars', WHITESPACE)
    source = re.sub(r'\s', '', replace_all(source, clear_chars))

    # 仅供参考，建议自行修改
    kwargs.update({
        'datetime': datetime.now().strftime(DATETIME_12),
        'spider': spider,
        'source': source,
        'region': replace_all(source, list(kwargs.get('city', '')))[:3] if spider[5] != '0' else '',
        'home_url': home_url,
        'author': kwargs.pop('author', 'White Turing'),
    })

    spiderfile = f'{dst}/{spider}.py'
    render_templatefile(spiderfile, templatefile, **kwargs)



'''
    常量，建议大写
'''

HEADERS = {
    'User-Agent': 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10.8; rv:61.0) Gecko/20100101 Firefox/61.0',
    'Accept': 'text/html,application/xhtml+xml,application/xml;q=0.9,image/webp,*/*;q=0.8',
    'Accept-Encoding': 'gzip, deflate, sdch',
    'Accept-Language': 'zh-CN,zh;q=0.8',
    'Connection': 'keep-alive',
}

PROXIES = {
    'http': 'http://127.0.0.1:8118',
    'https': 'https://127.0.0.1:8118',
}

TIMEOUT = 36

# 可处理的状态码
STATUS_CODES = [200, 521]

# 网页常见空白符号
WHITESPACE = ('\xa0', '\u3000', '\x0c', '&nbsp;', ' ')

# 格式化时间
DATETIME_12 = r'%Y-%m-%d %I:%M:%S %p'  # 2020-07-02 08:17:54 AM
DATETIME_24 = r'%Y-%m-%d %H:%M:%S'  # 2020-07-02 21:29:33



'''
    中文大写金额转换成阿拉伯数字金额
    请确保输入的中文是金额，这里不作验证
'''

CN_NUM = {
    '〇': 0, '一': 1, '二': 2, '三': 3, '四': 4, '五': 5, '六': 6, '七': 7, '八': 8, '九': 9, '零': 0, '壹': 1, '贰': 2, '叁': 3, '肆': 4, '伍': 5, '陆': 6, '柒': 7, '捌': 8, '玖': 9, '貮': 2, '两': 2,
}

CN_UNIT = {
    '分': 0.01,
    '角': 0.1,
    '元': 1,
    '整': 1,
    '十': 10,
    '拾': 10,
    '百': 100,
    '佰': 100,
    '千': 1000,
    '仟': 1000,
    '万': 10000,
    '萬': 10000,
    '亿': 100000000,
    '億': 100000000,
    '兆': 1000000000000,
}


def chinese_to_arabic(cn: str) -> int:
    unit = 0   # current
    ldig = []  # digest
    for cndig in reversed(cn):
        if cndig in CN_UNIT:
            unit = CN_UNIT.get(cndig)
            if unit == 10000 or unit == 100000000:
                ldig.append(unit)
                unit = 1
        else:
            dig = CN_NUM.get(cndig)
            if unit:
                dig *= unit
                unit = 0
            ldig.append(dig)
    if unit == 10:
        ldig.append(10)
    val, tmp = 0, 0
    for x in reversed(ldig):
        if x == 10000 or x == 100000000:
            val += tmp * x
            tmp = 0
        else:
            tmp += x
    val += tmp
    return round(val, 2)


if __name__ == "__main__":
    money = '贰佰陆拾柒万伍仟玖佰壹拾壹元肆角叁分'
    print(chinese_to_arabic(money))



'''
    常用操作的函数封装
'''

from functools import partial

from .constants import WHITESPACE


# 判断是否包含某些元素且不包含某些元素
def contains(string, within, without=tuple()) -> bool:
    if any(k in string for k in without):
        return False
    if any(k in string for k in within):
        return True
    return False


# 替换一组字符
def replace_all(string, words=tuple()) -> str:
    for w in words:
        string = string.replace(w, '')
    return string


# 替换空白符
replace_ws = partial(replace_all, words=WHITESPACE)



'''
    网络请求、解析等方法的工具库
'''

from .constants import DATETIME_12, DATETIME_24, HEADERS, WHITESPACE
from .formlib import extract_list, extract_string, is_table, parse_form
from .httplib import download, retry_get, retry_post, save_html
from .money import chinese_to_arabic
from .ocrlib import GeneralOcr
from .quick import contains, replace_all, replace_ws
